@@ -3,6 +3,7 @@ from typing import Callable
 import altair as alt
 import numpy as np
 import pandas as pd
+from altair.vegalite.v4.schema.channels import Color
 from pandas.api.types import (
     is_bool_dtype,
     is_categorical_dtype,
@@ -11,7 +12,7 @@ from pandas.api.types import (
     is_object_dtype,
 )
 
-from profiling.utils import vega_sanitize
+from profiling.utils import format_time, vega_sanitize
 
 
 def condexp(
@@ -32,6 +33,12 @@ def condexp(
         pass
 
 
+def flatten_index(df):
+    """Flatten hiearchical column index"""
+    df.columns = df.columns.to_flat_index()
+    return df
+
+
 def condexp_num(
     func: Callable[[pd.DataFrame], pd.Series],
     x: pd.DataFrame,
@@ -47,6 +54,7 @@ def condexp_num(
     pred = func(x).to_numpy()
 
     df = x.assign(pred=pred, y=y)
+
     if df[var].nunique() > n_cut:
         groupvar = df[var] + 1e-6 * np.random.uniform(size=len(df))
         df["groups"] = pd.qcut(groupvar, n_cut, duplicates="drop")
@@ -55,9 +63,17 @@ def condexp_num(
 
     df_act = (
         df.groupby("groups")
-        .agg({var: "mean", "y": "mean"})
+        .agg({var: "mean", "y": ["mean", "std", "count"]})
+        .pipe(flatten_index)
         .assign(__taktile_profiling_value_type__="Actual Value")
-        .rename(columns={"y": y.name})
+        .rename(
+            columns={
+                ("y", "mean"): y.name,
+                ("y", "std"): "y_std",
+                ("y", "count"): "y_count",
+                (var, "mean"): var,
+            }
+        )
     )
 
     df_pred = (
@@ -67,10 +83,28 @@ def condexp_num(
         .rename(columns={"pred": y.name})
     )
 
-    df = df_pred.append(df_act, ignore_index=True)
+    df = pd.concat([df_pred, df_act], ignore_index=True)
+
+    df_error = (
+        df_act.assign(
+            lower_bound=lambda x: x[y.name]
+            - (1.96 * x["y_std"] / np.sqrt(x["y_count"])),
+            upper_bound=lambda x: x[y.name]
+            + (1.96 * x["y_std"] / np.sqrt(x["y_count"])),
+            __taktile_profiling_value_type__=lambda x: "Actual Value",
+        )
+        .rename(columns={y.name: "y.name"})  # to avoid error with melt
+        .melt(
+            id_vars=[var, "__taktile_profiling_value_type__"],
+            value_vars=["lower_bound", "upper_bound"],
+            var_name="__taktile_actual_value_ci__",
+            value_name=y.name,
+        )
+    )
 
     # Create plot
     var_sanitized = vega_sanitize(var, datatype="Q")
+
     chart = (
         alt.Chart(df)
         .mark_line(point=True)
@@ -89,13 +123,18 @@ def condexp_num(
         .interactive()
     )
 
-    return chart
+    chart_error = (
+        alt.Chart(df_error)
+        .mark_errorband()
+        .encode(
+            x=alt.X(var_sanitized, scale=alt.Scale(zero=False)),
+            y=alt.Y(y.name, scale=alt.Scale(zero=False)),
+            color=alt.Color("__taktile_profiling_value_type__:N"),
+        )
+        .interactive()
+    )
 
-
-def flatten_index(df):
-    """Flatten hiearchical column index"""
-    df.columns = df.columns.to_flat_index()
-    return df
+    return chart + chart_error
 
 
 def condexp_cat(
@@ -112,7 +151,7 @@ def condexp_cat(
         x.assign(pred=pred, y=y)
         .astype({var: "category"})
         .groupby(var, as_index=False, observed=True)
-        .agg({"y": "mean", "pred": ["mean", "count"]})
+        .agg({"y": ["mean", "std", "count"], "pred": ["mean", "count"]})
         .pipe(flatten_index)
         .sort_values(("pred", "count"), ascending=False)
         .head(n_categories)  # keep most frequent
@@ -123,6 +162,8 @@ def condexp_cat(
                 ("y", "mean"): "Actual Value",
                 ("pred", "mean"): "Predicted Value",
                 ("pred", "count"): "N",
+                ("y", "std"): "y_std",
+                ("y", "count"): "y_count",
             }
         )
     )
@@ -137,6 +178,23 @@ def condexp_cat(
         __taktile_profiling_value_type__="Actual Value"
     )
 
+    df_error = (
+        df_act.assign(
+            lower_bound=lambda x: x[y.name]
+            - (1.96 * x["y_std"] / np.sqrt(x["y_count"])),
+            upper_bound=lambda x: x[y.name]
+            + (1.96 * x["y_std"] / np.sqrt(x["y_count"])),
+            __taktile_profiling_value_type__=lambda x: "Actual Value",
+        )
+        .rename(columns={y.name: "y.name"})  # to avoid error with melt
+        .melt(
+            id_vars=[var, "__taktile_profiling_value_type__"],
+            value_vars=["lower_bound", "upper_bound"],
+            var_name="__taktile_actual_value_ci__",
+            value_name=y.name,
+        )
+    )
+
     # Create plot
     var_sanitized = vega_sanitize(var, datatype="N")
 
@@ -146,14 +204,16 @@ def condexp_cat(
         .encode(
             x=alt.X(y.name, scale=alt.Scale(zero=False)),
             y=alt.Y(
-                var_sanitized, sort=ordered_categories, scale=alt.Scale(zero=False),
+                var_sanitized,
+                sort=ordered_categories,
+                scale=alt.Scale(zero=False),
             ),
             color=alt.Color(
                 "__taktile_profiling_value_type__:N", legend=alt.Legend(title="Type")
             ),
             tooltip=[
                 alt.Tooltip("__taktile_profiling_value_type__:N", title="Type"),
-                var_sanitized,
+                alt.Tooltip(var_sanitized),
                 alt.Tooltip(f"{y.name}:Q", format=".2f"),
             ],
         )
@@ -167,17 +227,31 @@ def condexp_cat(
             x=alt.X(y.name, scale=alt.Scale(zero=False)),
             y=alt.Y(var_sanitized, sort=ordered_categories),
             color=alt.Color(
-                "__taktile_profiling_value_type__:N", legend=alt.Legend(title="Type")
+                "__taktile_profiling_value_type__:N",
+                legend=alt.Legend(title="Type"),
             ),
             tooltip=[
                 alt.Tooltip("__taktile_profiling_value_type__:N", title="Type"),
-                var_sanitized,
+                alt.Tooltip(var_sanitized),
                 alt.Tooltip(f"{y.name}:Q", format=".2f"),
             ],
         )
         .interactive()
     )
-    chart = chart_pred + chart_act
+
+    chart_error = (
+        alt.Chart(df_error)
+        .mark_errorbar(ticks=True)
+        .encode(
+            x=alt.X(y.name, scale=alt.Scale(zero=False)),
+            y=alt.Y(
+                var_sanitized, sort=ordered_categories, scale=alt.Scale(zero=False)
+            ),
+            color=alt.Color("__taktile_profiling_value_type__:N"),
+        )
+        .interactive()
+    )
+    chart = chart_pred + chart_act + chart_error
 
     return chart
 
@@ -193,7 +267,7 @@ def condexp_datetime(func, X, y, var, n_max=100):
 
     df_agg = (
         df.groupby(var, as_index=False, observed=True)
-        .agg({"y": "mean", "pred": ["mean", "count"]})
+        .agg({"y": ["mean", "std", "count"], "pred": ["mean", "count"]})
         .pipe(flatten_index)
         .sort_values(("pred", "count"), ascending=False)
         .head(n_max)  # keep most frequent
@@ -203,6 +277,8 @@ def condexp_datetime(func, X, y, var, n_max=100):
                 ("y", "mean"): "Actual Value",
                 ("pred", "mean"): "Predicted Value",
                 ("pred", "count"): "N",
+                ("y", "std"): "y_std",
+                ("y", "count"): "y_count",
             }
         )
     )
@@ -214,11 +290,31 @@ def condexp_datetime(func, X, y, var, n_max=100):
         __taktile_profiling_value_type__="Actual Value"
     )
 
-    df = df_pred.append(df_act, ignore_index=True)
+    df = pd.concat([df_pred, df_act], ignore_index=True)
+
+    df_error = (
+        df_act.assign(
+            lower_bound=lambda x: x[y.name]
+            - (1.96 * x["y_std"] / np.sqrt(x["y_count"])),
+            upper_bound=lambda x: x[y.name]
+            + (1.96 * x["y_std"] / np.sqrt(x["y_count"])),
+            __taktile_profiling_value_type__=lambda x: "Actual Value",
+        )
+        .rename(columns={y.name: "y.name"})  # to avoid error with melt
+        .melt(
+            id_vars=[var, "__taktile_profiling_value_type__"],
+            value_vars=["lower_bound", "upper_bound"],
+            var_name="__taktile_actual_value_ci__",
+            value_name=y.name,
+        )
+    )
 
     # Create plot
     var_sanitized = vega_sanitize(var, datatype="T")
 
+    # Format datetimes correctly
+    formatted_time = format_time(X, var)
+
     chart = (
         alt.Chart(df)
         .mark_line(point=True)
@@ -230,11 +326,21 @@ def condexp_datetime(func, X, y, var, n_max=100):
             ),
             tooltip=[
                 alt.Tooltip("__taktile_profiling_value_type__:N", title="Type"),
-                alt.Tooltip(var_sanitized, format=".2f"),
+                alt.Tooltip(var_sanitized, format=formatted_time),
                 alt.Tooltip(f"{y.name}:Q", format=".2f"),
             ],
         )
         .interactive()
     )
+    chart_error = (
+        alt.Chart(df_error)
+        .mark_errorband()
+        .encode(
+            x=alt.X(var_sanitized, scale=alt.Scale(zero=False)),
+            y=alt.Y(y.name, scale=alt.Scale(zero=False)),
+            color=alt.Color("__taktile_profiling_value_type__:N"),
+        )
+        .interactive()
+    )
 
-    return chart
+    return chart + chart_error
